require 'hashie'

module Galactus
  class Response < Hash
    include Hashie::Extensions::MergeInitializer
    include Hashie::Extensions::MethodAccess
    include Hashie::Extensions::IndifferentAccess
    include Hashie::Extensions::KeyConversion

    attr_reader :raw_response

    def self.create(response)
      case response.code
        when 200
          Response.new(response)
        when 304
          NotModifiedResponse.new(response)
        else
          ErrorResponse.new(response)
      end
    end

    def initialize(response)
      @raw_response = response
      super(response)
    end
  end

  class ErrorResponse < Response; end

  class NotModifiedResponse < Response
    def initialize(response)
      super({
        status: 'Not Modified',
        code: 304,
        data: {},
        etag: response.headers['etag'] })
    end
  end
end
